<?php
/* parse_config.inc.php - Parse configuration file of Nagios
 * Part of NAUPY - Nagios Ain't Using PHP Yet
 * Copyright (C) <2003> Craig Small <csmall@small.dropbear.id.au>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
 
/*
 	Modified by Taylor J. Dondich
 	For use in Fruity
*/

/*
 	Modified by Cdric Temple - Linagora SA
 	For use in Nareto
*/

class Nagios {
  var $version = '0.03';
  var $config;
  var $contactgroups;
  var $contacts;
  var $hostgroups;
  var $hosts;
  var $services;
  var $host_status;
  var $service_status;
  var $errormsg = "";
  var $timeperiods;

  // Constructor
  function Nagios() 
  {
    $config = array();
    $contactgroups = array();
    $contacts = array();
    $hostgroups = array();
    $hosts = array();
    $services = array();
    $host_status = array();
    $service_status = array();
    $timeperiods = array();
    $commands = array();
  }

  ///////////// Public Functions /////////////////////

  // Parse nagios main configuration file $filename
  function ParseConfigFile($filename)
  {
    if ( ($fp = @fopen($filename, 'r')) == FALSE) {
      $this->errormsg =  "Cannot open config file $filename";
      return FALSE;
    }

    while ($line = fgets($fp, 987)) {
      if (preg_match('/^\s*(|#.*)$/', $line)) {
        continue;
      }

      if (preg_match('/^\s*cfg_file\s*=\s*(\S+)/', $line, $regs)) {
        if ($this->_ParseObjectFile($regs[1]) === FALSE) {
          $this->errormsg = "Cannot open config file ". $regs[1];
          return FALSE;
        }
        continue;
      }
      
      if (preg_match('/^\s*xedtemplate_config_file\s*=\s*(\S+)/', $line, $regs)) {
        if ($this->_ParseObjectFile($regs[1]) === FALSE) {
          $this->errormsg = "Cannot open config file ". $regs[1];
          return FALSE;
        }
        continue;
      }

      if (preg_match('/^\s*(\S+)\s*=\s*(\S+)/', $line, $regs)) {        
        $this->config[$regs[1]] = $regs[2];
        continue;
      }

    } //while

    return true;
  } // parse_config_file
  

  // Parse Nagios status file 
  // DO NOT USE : THERE IS A BUG FOR STATE
  // INFORMATION THIS FUNCTION DO NOT VERIFY IS THE SERVICE IS IN HARD
  // OR SOFT STATE
  function ParseStatusFile()
  {
    $hostst_fields = array ('status','last_check','last_state_change','problem_has_been_acknowledged','time_up','time_down','time_unreachable','(unsigned long)last_notification','current_notification_number','notifications_enabled','event_handler_enabled','checks_enabled','flap_detection_enabled','is_flapping','percent_state_change','scheduled_downtime_depth','failure_prediction_enabled','process_performance_data','plugin_output');
    $servicest_fields = array ('status','attempts','state_type','last_check','next_check','check_type','checks_enabled','accept_passive_service_checks','event_handler_enabled','last_state_change','problem_has_been_acknowledged','last_hard_state','time_ok','time_unknown','time_warning','time_critical','last_notification','current_notification_number','notifications_enabled','latency','execution_time','flap_detection_enabled','is_flapping','percent_state_change','scheduled_downtime_depth','failure_prediction_enabled','process_performance_data','obsess_over_service','plugin_output');

    if ( ($fp = fopen($this->config['status_file'], 'r')) == FALSE) {
      $this->errormsg =  "Could not open status file $filename";
      return FALSE;
    }

    while ($line = fgets($fp,987)) {
      if (preg_match('/\[(\d+)\] SERVICE;([^;]+);([^;]+);(.+)$/', $line, $regs)) {
        $tmparr = explode(';', $regs[4]);
        $this->service_status[$regs[2]][$regs[3]] = array ('last_update' => $regs[1], 'host_name' => $regs[2], 'description' => $regs[3]);
        foreach($servicest_fields as $ordinal => $key) {
          $this->service_status[$regs[2]][$regs[3]][$key] = $tmparr[$ordinal];
        }
        continue;
      } // SERVICE; ...
      
      if (preg_match('/\[(\d+)\] HOST;([^;]+);(.+)$/', $line, $regs)) {
        $tmparr = explode(';', $regs[3]);
        $this->host_status[$regs[2]] = array('last_update' => $regs[1], 'host_name' => $regs[2]);
        foreach($hostst_fields as $ordinal => $key) {
          $this->host_status[$regs[2]][$key] = $tmparr[$ordinal];
        }
        continue;
      } // HOST; ...
        
    } // WHILE
    fclose($fp);
  }



  // Get All HostGroups that are managed by specified ContactGroup
  function GetContactgroupHostgroups($cgroupname)
  {
    $hgroups = array();
    foreach($this->hostgroups as $hgroupname => $hgroupparms) {
      $members = explode(',', $hgroupparms['contact_groups']);

      if (in_array($cgroupname, $members)) {
        $hgroups[$hgroupname] = $this->hostgroups[$hgroupname];
      }

    }

    return $hgroups;
  }



  // Get all contactgroups that $contactname is member of
  function GetContactContactgroups($contactname)
  {
    $groups = array();
    foreach($this->contactgroups as $groupname => $groupparms) {
      $members = explode(',', $groupparms['members']);

      if (in_array($contactname, $members)) {
        $groups[$groupname] = $this->contactgroups[$groupname];
      }

    }

    return $groups;
  }



  // Get all hosts members of hostgroup $groupname
  function GetHostgroupMembers($groupname)
  {
    return explode(',', $this->hostgroups[$groupname]['members']);
  }



  // Get all hostgroups that $host is member of
  function GetHostsGroups($host)
  {
    $hosts_groups = array();

	if (is_array($this->hostgroups)) {
		foreach ($this->hostgroups as $groupname => $group) {
			$members = explode(',', $group['members']);

			if (in_array($host, $members)) {
				$hosts_groups[] = $groupname;
			}

		}
	}
    return $hosts_groups;
  }



  // Get the service status
  function GetServiceStatus($host, $service)
  {
    return $this->service_status[$host][$service];
  }



  // Get the hostname of the host that has address == $ipaddress
  function GetHostByAddr($ipaddr)
  {
    foreach($this->hosts as $hostname => $parms)
      {
        if ($parms['register'] == 0) continue;

        if ($parms['address'] == $ipaddr) {
          return $parms['host_name'];
        }

      }
    return FALSE;
  }

  
  
  // Return the array corresponding to service $service_description
  // associated to host $host_name
  function GetHostService($host_name, $service_description){
    $host_services = $this->GetHostServices($host_name);
    
    foreach ($host_services as $service) {
      if ( $service['service_description'] == $service_description ) {
        return $service;
      }
    }
    
    return FALSE;
  }



  // Get all of services linked to the host $host Services linked to
  // hostgroup that host $host is member of are returned
  function GetHostServices($host)
  {   
    $hostservices = array();
    $hosts_groups = $this->GetHostsgroups($host);
    
    if (is_array($this->services)) {
    foreach($this->services as $service => $parms)
      {
        if ($parms['register'] == '0') continue;

        $hosts = explode(',', $parms['host_name']);
        $services_groups = explode(',', $parms['hostgroup_name']);
            
        if (in_array($host, $hosts)) {              
          $hostservices[$service] = $parms;
        }

        foreach ($hosts_groups as $key => $host_group) {              
          if (in_array($host_group, $services_groups, TRUE)) {                    
            $hostservices[$service] = $parms;                
          }
        }        

      }
    }
    return $hostservices;
  }



  // Give the check_period of the service $service.
  // $service must be an array as defined in this class.
  // $service could be a template.
  // If the service has no check_period defined, we use the 
  // check_period of the template used by the service
  function give_timeperiod_name($service) {
    if ($service['check_period'] ) {
      return $service['check_period'];
    }
    // Search for template used by the service and return the check_period
    else {
    if (is_array($this->services)) { 
      foreach ($this->services as $key => $parms) {
        
        if ($parms['name'] == $service['use']) {
          return $this->give_timeperiod_name( $parms);
        }
      
      } // foreach
    }
    } // else

    return FALSE;
  }


  // Return the timeperiod $name
  function give_time_period_value($name){
  	if (is_array($this->timeperiods)) {
    foreach ($this->timeperiods as $key=>$tp) {
      if ( $tp['timeperiod_name'] == $name) {
        return $tp;
      }
    }
  	}
    return FALSE;
  }


  // Say if the service $service_description for host $host is in period of check
  function service_is_in_check_period($host_name, $service_description){
    // get current day
    $today = getdate();
    $current_time = date("H:s");
    // echo "current_time : $current_time\t";
    $localday = strtolower($today['weekday']);
    // echo "$localday\t";
    // get service
    $service = $this->GetHostService($host_name, $service_description);
    // get service check_period
    $timeperiod_array = $this->give_time_period_value( $this->give_timeperiod_name($service));
    $timeperiod_str = $timeperiod_array[ $localday ] ;
    // echo "$timeperiod_str\t";
    if ( !isset( $timeperiod_str) ) {
      return FALSE;
    }

    // parse time periodn we get an array of timechecks like "08:00-18:00"  
    $check_period_times = explode(',', $timeperiod_str);
    foreach ( $check_period_times as $check_period_time ) {
      $tmp = explode('-', $check_period_time);
      $begin_time = $tmp[ 0 ];
      $end_time = $tmp[ 1 ];
      // echo "$begin_time  $end_time\n";
      if ( ($current_time >= $begin_time)
           &&
           ($current_time <= $end_time)
           ) {
        return TRUE;
      }
      
      return FALSE;
    }
  }

  // parse files defined in Nagios main configuration file (cfg_file
  // directive)
  function _ParseObjectFile($filename)
  {
    $scanstate = '';

    if ( ($fp = @fopen($filename, 'r')) == FALSE) {
      $this->errormsg =  "Cannot open object file $filename";
      return FALSE;
           
    }

    

  ////////////// Private functions ///////////////////////////////////
  

    while ($line = fgets($fp, 987)) {	// Why are we limiting it to 987 in length?
      //echo "Debug: ($scanstate) $line<br>";
      if (preg_match('/^\s*(|#.*)$/', $line)) {
        continue;
      }

      if (preg_match('/^\s*define\s+(\S+)\s*{\s*$/', $line, $regs)) {
        $scanstate = $regs[1];
        $tmpobject = array();
        continue;
      }

      if (preg_match('/\s*}/', $line)) { //Completed object
        switch($scanstate) {
        case 'contactgroup':
          $this->contactgroups[] = $tmpobject;
          break;
        case 'contact':
          $this->contacts[] = $tmpobject;
          break;
        case 'host':
          $this->hosts[] = $tmpobject;
          break;
        case 'hostgroup':
          $this->hostgroups[] = $tmpobject;
          break;
        case 'timeperiod':
          $this->timeperiods[] = $tmpobject;
          break;
        case 'command':
          $this->commands[] = $tmpobject;
          break;
        case 'service':
          $this->services[] = $tmpobject;
          break;
        case 'servicegroup':
          $this->servicegroups[] = $tmpobject;
          break;
        case 'hostextinfo':
          $this->hostextinfo[] = $tmpobject;
          break;
        case 'serviceextinfo':
          $this->serviceextinfo[] = $tmpobject;
          break;
        } // switch
        $scanstate = '';
        continue;
      }

      if (preg_match('/\s*(\S+)\s+([\SA-a ]+)/', $line, $regs)) {
        /*
            	if ($regs[1] == 'use') {
                    $registered = isset($tmpobject['register']);
                    switch ($scanstate) {
                    case 'host':
                        $tmpobject = array_merge($this->hosts[$regs[2]],$tmpobject);
                        break;
                    case 'hostgroup':
                        $tmpobject = array_merge($this->hostgroups[$regs[2]],$tmpobject);
                        break;
                    case 'service':
                        $tmpobject = array_merge($this->services[$regs[2]],$tmpobject);
                        break;
                case 'command':
                    $tmpobject = array_merge($this->commands[$regs[2]],$tmpobject);
                    break;                      

                    }
                    if (!$registered) {
                        unset($tmpobject['register']);
                    }
                } else {
        */                
        $tmpobject[$regs[1]] = $regs[2];
        //                }
        continue;
      }
    } //while
  } // _ParseObjectFile


} // end of Class Nagios
// vim:et:sw=4:ts=4:
?>
