#!/usr/bin/perl

#    nagios_alert.pl
#
#    This file is part of the NaReTo project
#
#    Copyright (C) 2005 Cdric Temple Linagora SA
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use DBI;
use POSIX;
use Config::IniFiles;

my $file = '/var/www/html/nareto/include/config.ini.php';
my $cfg ;

if ( -f $file ) {
     $cfg = new Config::IniFiles( -file => $file, -commentchar => '/' );
 } else {
     die "The configuration file $file was not found";
 }

# PATH to nagios.log archives files
#   default is '/var/log/nagios/archives'
my $nag_arch_path = $cfg->val('Nagios Configuration','archives_path');



my $DEBUG = 0;
my ($DB_User, $DB_Host, $DB_Pass, $DB_Name) = ($cfg->val('Mysql Configuration','user'), 
                                               $cfg->val('Mysql Configuration','host'), 
                                               $cfg->val('Mysql Configuration','password'),
                                               $cfg->val('Mysql Configuration','nareto_db'));
my $dsn = "DBI:mysql:database=$DB_Name;host=$DB_Host";
my $dbh = DBI->connect($dsn, $DB_User, $DB_Pass, {'RaiseError' => 1});

if ( defined($ARGV[0]) ) {
    parse_file( $ARGV[0] );
    exit (0);
}

my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
    localtime();

my $yesterday = strftime( '%m-%d-%Y', $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);


my @files = glob($nag_arch_path.'/nagios-'.$yesterday.'-??.log');

foreach my $i (@files) {
    parse_file( $i );
    
}

# close the alert for service $servicename on host $hostname
# $timestamp is the time the alert is closed
sub close_alert{
    my $timestamp = shift;
    my $hostname = shift;
    my $servicename = shift;
#    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
#        localtime($timestamp);
    my $time_corrige=$timestamp;
    my $sql = "UPDATE suivi_semaine SET time_corrige='".$time_corrige.
        "' WHERE host='".$hostname."' AND service='".$servicename.
        "' AND time_corrige=0";
    print "SQL: $sql \n" if ($DEBUG);
    my $sth = $dbh->prepare( $sql );
    $sth->execute();
}

# said if yes or no the alert exists in the database
# a alert exists if it is present in the database
# and it is not already corrected
sub alert_exist{
    my $hostname = shift;
    my $servicename = shift;
    my $sql = "SELECT id FROM suivi_semaine WHERE ".
        "host='".$hostname."' AND service='".$servicename.
        "' AND time_corrige='0'";
    print "SQL: $sql \n" if ($DEBUG);
    my $sth = $dbh->prepare( $sql );
    $sth->execute();
    my $result =   $sth->fetchrow_hashref();
    return $result->{'id'};
    
}

# add alert in the database
sub add_alert{
    my $timestamp = shift;
    my $hostname = shift;
    my $servicename = shift;
    my $state = shift;
    my $comment = shift;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
        localtime($timestamp);
    my $time_str = $timestamp;
    my $jour = strftime('%Y-%m-%d', $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
    my $sql = 'INSERT INTO suivi_semaine(timestamp, jour, host, '.
        'service, commentaire, etat, time_corrige, time_ack) VALUES ('.
        $time_str.", '".$jour."' , '".$hostname."' , '".
        $servicename."' , '".$comment."' , '".substr($state,0,1).
        "' , 0, 0)";
    my $sth = $dbh->prepare( $sql );
    $sth->execute();
}

sub get_alert_state{
    my $id = shift;
    my $sql = "SELECT etat FROM suivi_semaine WHERE id=".$id.";";
    my $sth = $dbh->prepare( $sql );
    $sth->execute();
    my $result =   $sth->fetchrow_hashref();
    return $result->{'etat'};
}

# said if current_state is bigger than old_state
sub gravity_cmp{
    my $current_state = shift;
    my $old_state = shift;
    my $result = 0 ;
    
    if ( $current_state eq $old_state ) {
        $result = 0;
    }
    elsif ( $current_state eq 'C' ) {
        $result = 1;
    }
    elsif ( ( $current_state eq 'W' ) &&
            ( $old_state eq 'U')
            ) {
        $result = 1;
    }
    else {
        $result = 0;
    }
    
    return $result;
}

sub alert_update_state{
    my $alert_id = shift;
    my $current_state = shift;
    my $sql = "UPDATE suivi_semaine set etat='".$current_state."' WHERE id=".$alert_id;
    my $sth = $dbh->prepare( $sql );
    $sth->execute();
}

sub alert{
    my $timestamp = shift;
    my $hostname = shift;
    my $servicename = shift;
    my $state = shift;
    my $comment = shift;
    
    my $alert_id; 
    if ($alert_id = alert_exist( $hostname, $servicename)) {
        print "alert exist\n" if ($DEBUG);
        my $old_state = get_alert_state( $alert_id ) ;
        my $current_state = substr( $state, 0, 1);
        if ( gravity_cmp( $current_state, $old_state) ) {
            alert_update_state( $alert_id, $current_state) ;
        }
    }
    else {
        print "alert NOT exist\n" if ($DEBUG);
        add_alert( $timestamp, $hostname, $servicename, $state, $comment);
    }
}

sub max{
    my $a = shift;
    my $b = shift;
    if ( $a > $b ) {
        return $a;
    }
    else {return $b;}
}

sub get_more_recent_timestamp{
    my $sql = "SELECT MAX(timestamp) FROM suivi_semaine";
    my $sth = $dbh->prepare( $sql );
    $sth->execute();

    my $result1 = $sth->fetchrow_hashref()->{'MAX(timestamp)'};
    
    $sql = "SELECT MAX(time_corrige) FROM suivi_semaine";
    $sth = $dbh->prepare( $sql );
    $sth->execute();

    my $result2 = $sth->fetchrow_hashref()->{'MAX(time_corrige)'};

    $sql = "SELECT MAX(time_ack) FROM suivi_semaine";
    $sth = $dbh->prepare( $sql );
    $sth->execute();

    my $result3 = $sth->fetchrow_hashref()->{'MAX(time_ack)'};

    
    my $result = max($result3, max($result1,$result2));
    

    return $result;

}

sub ack_service{
    my $timestamp = shift;
    my $hostname = shift;
    my $servicename = shift;
    my $user = shift;
    my $comment = shift;
       
    my $sql = "UPDATE suivi_semaine SET time_ack='".$timestamp."', ackeur='".$user
        ."', comm_ack='".$comment."' WHERE time_corrige=0 AND time_ack=0".
        " AND host='".$hostname."' AND service='".$servicename."'";

    my $sth = $dbh->prepare( $sql );
    $sth->execute();
}

sub parse_file{
    my $file = shift;
    printf "Try to open file $file..." if ($DEBUG);
    if ( -f $file && -r $file ) {
        printf " OK\n" if ($DEBUG);
        open( FILE, "< $file");
        my $more_recent_timestamp = get_more_recent_timestamp();
        while (<FILE>) {
	    # [1153778406] SERVICE ALERT: ROT_FRS060;Ping_Host_Status;WARNING;SOFT;1;PING WARNING - Packet loss = 0%, RTA = 1087.98 ms
            # [1144676042] SERVICE ALERT: Firewall_Lille_1;Linux_interface_bandwith_INTRA;CRITICAL;HARD;3;(Return code of 25 is out of bounds)
            if ( /\[(\d+)\] SERVICE ALERT: (.+);(.+);(.+);(.+);(\d+);(.+)/ ) {
            
    
                printf "time: $1  host $2  service $3 state: $4 \n" if ($DEBUG > 2);
                if ( $1 <= $more_recent_timestamp) { next;}
		my $timestamp = $1;
                my $hostname = $2;
                my $servicename = $3;
                my $state = $4;
                my $criticity = $5;
                my $comment = $7;
                $comment =~ s/\'/ /g;
                $comment =~ s/\"/ /g;

                if ($state =~ /OK/) {
                    close_alert( $timestamp, $hostname, $servicename);
                }
                elsif (! ( 
                           ($cfg->val('Nareto Configuration', 'soft_alarm') == 0) 
                            &&
                            ($criticity eq 'SOFT')
                            
                           )
                       ) {
                    alert( $timestamp, $hostname, $servicename, $state, $comment);
                }
                
            }
            if ( /\[(\d+)\] EXTERNAL COMMAND: ADD_SVC_COMMENT;(.+);(.+);\d+;(.+);ACKNOWLEDGEMENT: (.+)/ ) {
                my $timestamp = $1;
                my $hostname = $2;
                my $servicename = $3;
                my $user = $4;
                my $comment = $5;
                $comment =~ s/\'/ /g;
                $comment =~ s/\"/ /g;
                ack_service( $timestamp, $hostname, $servicename,$user, $comment );
            }
        }
    }
    else {
        printf " Error: file $file not exists or not readable!\n";
    }
}
