#!/usr/bin/perl

#    nagios_alert_agregation,.pl
#
#    This file is part of the NaReTo project
#
#    Copyright (C) 2005 Cdric Temple Linagora SA
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


use strict;
use DBI;
use POSIX;
use Config::IniFiles;

my $file = '/var/www/html/nareto/include/config.ini.php';
my $cfg ;

if ( -f $file ) {
     $cfg = new Config::IniFiles( -file => $file, -commentchar => '/' );
 } else {
     die "The configuration file $file was not found";
 }



my $DEBUG = 3;
my ($DB_User, $DB_Host, $DB_Pass, $DB_Name) = ($cfg->val('Mysql Configuration','user'), 
                                               $cfg->val('Mysql Configuration','host'), 
                                               $cfg->val('Mysql Configuration','password'),
                                               $cfg->val('Mysql Configuration','nareto_db'));
my $dsn = "DBI:mysql:database=$DB_Name;host=$DB_Host";
my $dbh = DBI->connect($dsn, $DB_User, $DB_Pass, {'RaiseError' => 1});

if ( defined($ARGV[0]) ) {
    # first: clean all "ghost alarms"
    clean_alarms();
    clean_service_alarms();

    agregate( $ARGV[0] );
    exit (0);
}


my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();

my $yesterday = strftime( '%Y-%m-%d', $sec,$min,$hour,$mday-1,$mon,$year,$wday,$yday,$isdst);
my $now = mktime( $sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);



# first: clean all "ghost alarms"
clean_alarms();
clean_service_alarms();

# then agregate
agregate( $yesterday );


# a "ghost alarm" is generate when a host is created in Nagios, then a alarm is opened and the host is suppressed.
# In the log, the host suppression is NOT mentionned so the alarm is NEVER closed
# we have to do the job!
sub clean_alarms{
    
    my %host;
    my %service;
    # suppress all ghosts based on host
    # get all hosts and create an hash table
    my $sql_select_hosts = "SELECT name FROM hosts";
    my $sth_select_hosts = $dbh->prepare( $sql_select_hosts );
    $sth_select_hosts->execute();
    while ( my $host_name = $sth_select_hosts->fetchrow_hashref() ) {
	$host{$host_name->{name}} = 1;
    }
    # suppress alarms with no hosts in table hosts:
    $sql_select_hosts = "SELECT DISTINCT(host) AS host FROM suivi_semaine";
    $sth_select_hosts = $dbh->prepare( $sql_select_hosts );
    $sth_select_hosts->execute();
    while ( my $host_name = $sth_select_hosts->fetchrow_hashref() ) {
	if ( !defined($host{$host_name->{host}})) {
	    my $sql_delete = "DELETE FROM suivi_semaine WHERE host='".$host_name->{host}."'";
	    my $sth_delete = $dbh->prepare($sql_delete);
	    $sth_delete->execute();
	    if ( $sth_delete->err ) {
		printf "Error when deleting " .$sql_delete. "; error is : " . $sth_delete->errstr ."\n";
	    }
	}
    }
    

}


# In the log, the host suppression is NOT mentionned so the alarm is NEVER closed
# we have to do the job!
sub clean_service_alarms{

    my %host;
    my %service;
    # suppress all ghosts based on host
    # get all hosts and create an hash table
    my $sql_select_hosts = "SELECT name FROM services";
    my $sth_select_hosts = $dbh->prepare( $sql_select_hosts );
    $sth_select_hosts->execute();
    while ( my $host_name = $sth_select_hosts->fetchrow_hashref() ) {
        $host{$host_name->{name}} = 1;
    }
    # suppress alarms with no hosts in table hosts:
    $sql_select_hosts = "SELECT DISTINCT(service) AS service FROM suivi_semaine";
    $sth_select_hosts = $dbh->prepare( $sql_select_hosts );
    $sth_select_hosts->execute();
    while ( my $host_name = $sth_select_hosts->fetchrow_hashref() ) {
        if ( !defined($host{$host_name->{service}})) {
            my $sql_delete = "DELETE FROM suivi_semaine WHERE service='".$host_name->{service}."'";
            my $sth_delete = $dbh->prepare($sql_delete);
            $sth_delete->execute();
            if ( $sth_delete->err ) {
                printf "Error when deleting " .$sql_delete. "; error is : " . $sth_delete->errstr ."\n";
            }
        }
    }


}





sub agregate {
    my $day = shift;
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
    $sec = $min = $hour = 0;
    if ( $day =~ /(\d\d\d\d)-(\d\d)-(\d\d)/ ) {
        $year = $1 - 1900;
        $mon = $2 - 1;
        $mday = $3 + 1;
    }
    printf "day: $day\tnow: ".localtime(mktime($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst))."\n";
    printf "now: ".time."\tmktime: ".mktime($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)."\n";
    my $now = mktime($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
    my $sql_select_hosts = "SELECT DISTINCT(host) AS host FROM suivi_semaine";
    print "SQL SELECT HOSTS: $sql_select_hosts \n" if ($DEBUG);
    my $sth_select_hosts = $dbh->prepare( $sql_select_hosts );
    $sth_select_hosts->execute();
    while ( my $host_name = $sth_select_hosts->fetchrow_hashref() ) {
	my $sql_select_service = "SELECT DISTINCT(service) AS service FROM suivi_semaine WHERE host='".$host_name->{host}."'";
	print "SQL SELECT SERVICES: $sql_select_service \n" if ($DEBUG);
	my $sth_select_service = $dbh->prepare( $sql_select_service );
	$sth_select_service->execute();
	while ( my $service_name = $sth_select_service->fetchrow_hashref() ) {
	    my $sql_select_alarms = "SELECT  timestamp, time_ack, time_corrige FROM suivi_semaine " .
		"WHERE host='".$host_name->{host}."' AND service='".$service_name->{service}."' AND jour = '".$day."' ";
	    print "SQL SELECT ALARMS: $sql_select_alarms \n" if ($DEBUG);
	    my $sth_select_alarms = $dbh->prepare( $sql_select_alarms );
	    $sth_select_alarms->execute();
	    # calculate nb_alarm, moy_ack and moy_corrige
	    my $nb_alarmes = 0;
	    my $moy_ack = 0;
	    my $moy_corrige = 0;
	    
	    while ( my $alarms = $sth_select_alarms->fetchrow_hashref() ) {
		$nb_alarmes++;
		my ($value_ack, $value_corrige);
		
		if ( (defined($alarms->{time_corrige}) )
                     && ( $alarms->{time_corrige} > 0 )
                     ) {
                    $value_corrige = $alarms->{time_corrige};
                }
                else {
                    $value_corrige = $now;
                }
		if ( (defined($alarms->{time_ack}) )
		     && ( $alarms->{time_ack} > 0 )
		     ) {
		    $value_ack = $alarms->{time_ack};
		}
		else {
		    $value_ack = $value_corrige;
		}
		$moy_ack += ($value_ack - $alarms->{timestamp});
		$moy_corrige += ($value_corrige - $alarms->{timestamp});
		if ( $moy_ack < 0 ) {
		    printf "ERROR: moy_ack < 0 for ".$host_name->{host}." ".$service_name->{service}." ".localtime($alarms->{timestamp})."\n";
		}
		if ( $moy_corrige < 0 ) {
		    printf "ERROR: moy_corrige < 0 for ".$host_name->{host}." ".$service_name->{service}." ".localtime($alarms->{timestamp})."\n";
		}
	    }
            if ( $nb_alarmes > 0 ) {
                $moy_ack /= $nb_alarmes;
                $moy_corrige /= $nb_alarmes;
                # add entry into suivi_mois
                my $sql_insert = "INSERT INTO suivi_mois (date, host, service, nb_alarmes, moy_ack, moy_corrige) VALUES ('".
                    $day."', '". $host_name->{host} . "' , '".$service_name->{service}."', ".$nb_alarmes.", ".$moy_ack.", ".$moy_corrige.")";
                my $sth_insert = $dbh->prepare($sql_insert);
                $sth_insert->execute();
                if ( $sth_insert->err ) { 		
                    printf "Error when inserting " .$sql_insert. "; error is : " . $sth_insert->errstr ."\n";
                }
            }

	}
    }

    #my $sql_delete = "DELETE FROM suivi_semaine WHERE jour <= '".$day."' AND time_corrige>0";
    #my $sth_delete = $dbh->prepare($sql_delete);
    #$sth_delete->execute();
    #if ( $sth_delete->err ) {
    #printf "Error when deleting " .$sql_delete. "; error is : " . $sth_delete->errstr ."\n";
    #}

}
