/*
    parser_log.php

    This file is part of the NaReTo project.

    Copyright (C) 2006/2007 Cedric TEMPLE Linagora SA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../include/config.h"
#include "../include/locations.h"
#include "../include/common.h"
#include "../include/objects.h"
#include "../include/statusdata.h"

#include "../include/getcgi.h"
#include "../include/cgiutils.h"
#include "../include/cgiauth.h"


#include <mysql/mysql.h>

#define DEBUG 0


void reset_changed();
void make_links();
void recalcul_liaisons();
void delete_not_changed();
void parse_cfg();
void print_help( char * );

extern char   main_config_file[MAX_FILENAME_LENGTH];

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;
extern host *host_list;
extern service *service_list;
extern contact *contact_list;
extern contactgroup *contactgroup_list;


int result;

MYSQL db_mysql;
char * query;

int main(int argc, char ** ARGV){

  char * mysql_database,*  mysql_host, * mysql_user, *mysql_password;
  int mysql_port;

  if ( argc != 6 ) {
    print_help( ARGV[0] );
  }

  query = calloc(1000, sizeof(char));

  /* read the CGI configuration file */
  result=read_cgi_config_file(get_cgi_config_location());
  if(result==ERROR){
    cgi_config_file_error(get_cgi_config_location());
    printf("Ne peut lire %s\n", get_cgi_config_location());
    fflush(stdout);
    return ERROR;
  }
  

  /* read the main configuration file */
  result=read_main_config_file(main_config_file);
  if(result==ERROR){
    printf("Ne peut lire %s\n", main_config_file);
    fflush(stdout);
    return ERROR;
  }
  
  /* read all object configuration data */
  result=read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);
  if(result==ERROR){
    printf("Ne peut lire toute la configuration\n");
    fflush(stdout);
    return ERROR;
  }

  /* read all status data */
  result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
  if(result==ERROR){
    printf("Ne peut lire tous les etats\n");
    fflush(stdout);
    free_memory();
    return ERROR;
  }

  /* try to initialize MySQL */
  if (!mysql_init (&db_mysql)) {
    printf( "Cannot initialise MySQL\n");
    return ERROR;
  }

  mysql_host =  (char * ) ARGV[1]; 
  mysql_database = (char * )ARGV[2];
  mysql_user = (char * )ARGV[3];
  mysql_password = (char * )ARGV[4];
  mysql_port = atoi( (char * )ARGV[5] );

  /* try to connect to MySQL */
  if (!mysql_real_connect(&db_mysql, 
                          mysql_host, 
                          mysql_user, 
                          mysql_password, 
                          mysql_database, 
                          mysql_port,
                          NULL, 0)) {
    printf( "Can not connect to database \"%s\" on server \"%s:%d\", with user \"%s\" and password!\n",
            mysql_database,
            mysql_host, 
            mysql_port,
            mysql_user
             );
    fflush( stdout) ;
    return ERROR;    
  }

  reset_changed();
#ifdef DEBUG
  printf("fin reset_changed()\n"); fflush( stdout );
#endif

  parse_cfg();
#ifdef DEBUG
  printf("fin parse_cfg()\n"); fflush( stdout );
#endif

  
  make_links();
#ifdef DEBUG
  printf("fin make_links()\n"); fflush( stdout );
#endif

  recalcul_liaisons();
#ifdef DEBUG
  printf("fin recalcul_liaison()\n"); fflush( stdout );
#endif

  delete_not_changed();
#ifdef DEBUG
  printf("fin delete_not_changed()\n"); fflush( stdout );
#endif
                                          
  mysql_close( &db_mysql );

  return 0;

}


/**
 * function local_contact_exists
 * says if contact "contact_name" exists in database
 *
*/
int local_contact_exists( char * contact_name ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW contact;
  int contact_count;

#ifdef DEBUG
  printf( "contact_exists %s ???\n", contact_name);
#endif  

  snprintf(query, 1000, "SELECT COUNT(*) FROM contacts WHERE name='%s'", contact_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif
  mysql_query(&db_mysql, query);
  res = mysql_store_result(&db_mysql) ;
      
  contact = mysql_fetch_row(res);
  contact_count = atoi(contact[0]);
  free( query );
  
#ifdef DEBUG
  printf( "RESULT: %d\n", contact_count);
#endif

  return contact_count;
}


/**
 * function local_add_contact
 * add contact "contact_name" in database
 *
*/
void local_add_contact( char * contact_name, char * contact_alias ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "add_contact %s %s \n", contact_name, contact_alias);
#endif  

  snprintf(query, 1000, "INSERT INTO contacts (name, alias, changed) VALUES('%s', '%s', '2')", contact_name, contact_alias );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);

  
}

/**
 * function local_update_contact
 * update contact "contact_name" in database
 *
*/
void local_update_contact( char * contact_name, char * contact_alias ) {

  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "update_contact %s %s \n", contact_name, contact_alias);
#endif  

  snprintf(query, 1000, "UPDATE contacts SET alias='%s', changed='1' WHERE name='%s'", contact_alias, contact_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);
  
}

/**
 * function parse_contact
 * put all new contacts in table contacts
 *
*/
void parse_contact() {

  contact *temp_contact=NULL;
  

#ifdef DEBUG
  printf("\n\nparse_contact\n");
#endif

  for(temp_contact=contact_list;temp_contact!=NULL;temp_contact=temp_contact->next){
#ifdef DEBUG
    printf( "CONTACT NAME: %s\n", temp_contact->name);
#endif
    if ( local_contact_exists( temp_contact->name ) ) {
      local_update_contact( temp_contact->name, temp_contact->alias );
    }
    else {
      local_add_contact( temp_contact->name, temp_contact->alias );
    }
  }
  
}







/**
 * function local_host_exists
 * says if host "host_name" exists in database
 *
*/
int local_host_exists( char * host_name ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW host;
  int host_count;

#ifdef DEBUG
  printf( "host_exists %s ???\n", host_name);
#endif  

  snprintf(query, 1000, "SELECT COUNT(*) FROM hosts WHERE name='%s'", host_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif
  mysql_query(&db_mysql, query);
  res = mysql_store_result(&db_mysql) ;
      
  host = mysql_fetch_row(res);
  host_count = atoi(host[0]);
  free( query );
  
#ifdef DEBUG
  printf( "RESULT: %d\n", host_count);
#endif

  return host_count;
}


/**
 * function local_add_host
 * add host "host_name" in database
 *
*/
void local_add_host( char * host_name, char * host_alias ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "add_host %s %s \n", host_name, host_alias);
#endif  

  snprintf(query, 1000, "INSERT INTO hosts (name, alias, register, changed) VALUES('%s', '%s', 1, 2)", host_name, host_alias );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);

  
}

/**
 * function local_update_host
 * update contact "host_name" in database
 *
*/
void local_update_host( char * host_name, char * host_alias ) {

  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "update_host %s %s \n", host_name, host_alias);
#endif  

  snprintf(query, 1000, "UPDATE hosts SET alias='%s', changed='1' WHERE name='%s'", host_alias, host_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);
  
}

/**
 * function parse_contact
 * put all new contacts in table contacts
 *
*/
void parse_host() {

  host * temp_host=NULL;
  

#ifdef DEBUG
  printf("\n\nparse_host\n");
#endif

  for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
#ifdef DEBUG
    printf( "host NAME: %s\n", temp_host->name);
#endif
    if ( local_host_exists( temp_host->name ) ) {
      local_update_host( temp_host->name, temp_host->alias );
    }
    else {
      local_add_host( temp_host->name, temp_host->alias );
    }
  }
  
}








/**
 * function local_contactgroup_exists
 * says if contactgroup "contactgroup_name" exists in database
 *
*/
int local_contactgroup_exists( char * contactgroup_name ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW contactgroup;
  int contactgroup_id;

#ifdef DEBUG
  printf( "contactgroup_exists %s ???\n", contactgroup_name);
#endif  

  snprintf(query, 1000, "SELECT id FROM contactgroups WHERE name='%s'", contactgroup_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif
  mysql_query(&db_mysql, query);
  res = mysql_store_result(&db_mysql) ;
      
  contactgroup = mysql_fetch_row(res);
  if ( contactgroup == NULL ) {
    return 0;
  }
  else {
    contactgroup_id = atoi(contactgroup[0]);
    free( query );
  
#ifdef DEBUG
    printf( "RESULT: %d\n", contactgroup_id);
#endif

    return contactgroup_id;
  }
}


/**
 * function local_add_contact
 * add contact "contact_name" in database
 *
*/
int local_add_contactgroup( char * contactgroup_name, char * contactgroup_alias ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "add_contactgroup %s %s \n", contactgroup_name, contactgroup_alias);
#endif  

  snprintf(query, 1000, "INSERT INTO contactgroups (name, alias, changed) VALUES('%s', '%s', 2)", contactgroup_name, contactgroup_alias );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);

  return mysql_insert_id( &db_mysql );
  
}

/**
 * function local_update_contact
 * update contact "contact_name" in database
 *
*/
void local_update_contactgroup( char * contactgroup_name, char * contactgroup_alias ) {

  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "update_contactgroup %s %s \n", contactgroup_name, contactgroup_alias);
#endif  

  snprintf(query, 1000, "UPDATE contactgroups SET alias='%s', changed='1' WHERE name='%s'", contactgroup_alias, contactgroup_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);
  
}





/**
 * function add_contacts_to_contactgroup
 * do the link between contacts and contactgroups
 *
*/
void add_contacts_to_contactgroup(int contactgroup_id, contactgroupmember * members) {
  contactgroupmember * member ;

  for( member = members; member != NULL; member = member->next ) {
    char * query = calloc(1000, sizeof(char));
    MYSQL_RES *res = NULL;
    MYSQL_ROW contact;
    int contact_count;
    
#ifdef DEBUG
    printf( "contact_exists_in_temp_table %s ???\n", member->contact_name);
#endif  

    snprintf(query, 1000, "SELECT COUNT(*) FROM l_temp WHERE id_temp='%d' AND value='%s' AND type='contact'", contactgroup_id, member->contact_name );
#ifdef DEBUG
    printf( "QUERY: %s\n", query);
#endif
    mysql_query(&db_mysql, query);
    res = mysql_store_result(&db_mysql) ;
    
    contact = mysql_fetch_row(res);
    contact_count = atoi(contact[0]);
    free( query );
    
#ifdef DEBUG
    printf( "RESULT: %d\n", contact_count);
#endif
    
    if ( contact_count == 0 ) {
#ifdef DEBUG
      printf( "CONTACT DOES NOT EXIST, we add it!\n");
#endif
      query = calloc(1000, sizeof(char));
      snprintf(query, 1000, "INSERT INTO l_temp SET id_temp='%d', value='%s', type='contact'", contactgroup_id, member->contact_name );
#ifdef DEBUG
      printf( "QUERY: %s\n", query);
#endif
      mysql_query(&db_mysql, query);
      if ( mysql_errno( &db_mysql )) {
        printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
      }
    }

  }

}

/**
 * function parse_contactgroup
 * put all new contactgroups in table contactgroups
 *
*/
void parse_contactgroup() {

  contactgroup *temp_contactgroup=NULL;
  int contactgroup_id = 0;

#ifdef DEBUG
  printf("\n\nparse_contact_group\n");
#endif

  for(temp_contactgroup=contactgroup_list;temp_contactgroup!=NULL;temp_contactgroup=temp_contactgroup->next){
    contactgroup_id = 0;
#ifdef DEBUG
    printf( "CONTACTGROUP NAME: %s\n", temp_contactgroup->group_name);
#endif
    if ( (contactgroup_id = local_contactgroup_exists( temp_contactgroup->group_name )) != 0 ) {
      local_update_contactgroup( temp_contactgroup->group_name, temp_contactgroup->alias );
    }
    else {
      contactgroup_id = local_add_contactgroup( temp_contactgroup->group_name, temp_contactgroup->alias );
    }

    add_contacts_to_contactgroup( contactgroup_id, temp_contactgroup->members );
  }
  
}












/**
 * function local_service_exists
 * says if service "service_name" exists in database
 *
*/
int local_service_exists( char * service_name ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW service;
  int service_id;

#ifdef DEBUG
  printf( "service_exists %s ???\n", service_name);
#endif  

  snprintf(query, 1000, "SELECT id FROM services WHERE name='%s'", service_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif
  mysql_query(&db_mysql, query);
  res = mysql_store_result(&db_mysql) ;
      
  service = mysql_fetch_row(res);
  if ( service == NULL ) {
    return 0;
  }
  else {
    service_id = atoi(service[0]);
    free( query );
  
#ifdef DEBUG
    printf( "RESULT: %d\n", service_id);
#endif

    return service_id;
  }
}


/**
 * function local_add_service
 * add contact "service_name" in database
 *
*/
int local_add_service( char * service_name, char * service_alias ) {
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "add_service %s %s \n", service_name, service_alias);
#endif  

  snprintf(query, 1000, "INSERT INTO services (name, alias, register, changed) VALUES('%s', '%s', 1, 2)", service_name, service_alias );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);

  return mysql_insert_id( &db_mysql );
}

/**
 * function local_update_service
 * update contact "service_name" in database
 *
*/
void local_update_service( char * service_name, char * service_alias ) {

  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  int error = 0;

#ifdef DEBUG
  printf( "update_service %s %s \n", service_name, service_alias);
#endif  

  snprintf(query, 1000, "UPDATE services SET alias='%s', changed='1' WHERE name='%s'", service_alias, service_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif

  error = mysql_query(&db_mysql, query);

  if ( mysql_errno( &db_mysql )) {
    printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
  }

  if ( error != 0 ) { exit(error);}

  free(query);
  
}


/**
 * function add_service_to_host
 * do the link between hosts and services
 *
*/
void add_service_to_host(char * host_name, char * service_description, int service_id) {
  
  char * query = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW service;
  int service_count;
  
#ifdef DEBUG
  printf( "service_exists_in_temp_table %s ???\n", service_description);
#endif  

  snprintf(query, 1000, "SELECT COUNT(*) FROM l_temp WHERE id_temp='%d' AND value='%s' AND type='service'", service_id, host_name );
#ifdef DEBUG
  printf( "QUERY: %s\n", query);
#endif
  mysql_query(&db_mysql, query);
  res = mysql_store_result(&db_mysql) ;
    
  service = mysql_fetch_row(res);
  service_count = atoi(service[0]);
  free( query );
    
#ifdef DEBUG
  printf( "RESULT: %d\n", service_count);
#endif
    
  if ( service_count == 0 ) {
#ifdef DEBUG
    printf( "SERVICE DOES NOT EXIST, we add it!\n");
#endif
    query = calloc(1000, sizeof(char));
    snprintf(query, 1000, "INSERT INTO l_temp SET id_temp='%d', value='%s', type='service'", service_id, host_name );
#ifdef DEBUG
    printf( "QUERY: %s\n", query);
#endif
    mysql_query(&db_mysql, query);
    if ( mysql_errno( &db_mysql )) {
      printf("Erreur Mysql: %s\n", mysql_error(&db_mysql) );
    }
  }
  

}

/**
 * function parse_service
 * put all new services in table services
 *
*/
void parse_service() {

  service *temp_service=NULL;
  int service_id = 0;

#ifdef DEBUG
  printf("\n\nparse_service\n");
#endif

  for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
    service_id = 0;
#ifdef DEBUG
    printf( "SERVICE NAME: %s\n", temp_service->description);
#endif
    if ( (service_id = local_service_exists( temp_service->description )) != 0 ) {
      local_update_service( temp_service->description, temp_service->description );
    }
    else {
      local_add_service( temp_service->description, temp_service->description );
      service_id = local_service_exists( temp_service->description );
    }

    add_service_to_host( temp_service->host_name, temp_service->description, service_id );
  }
  
}





/**
 * function parse_cfg
 * put all objects in tables
 *
*/
void parse_cfg() {
  parse_contact();
  parse_contactgroup();
  parse_host();
  /*parse_hostgroup();*/
  parse_service();

}


/**
 * function reset_changed
 * 
 * vide la table de stockage des liaisons entre objet et
 * rinitialise le champ changed a 0, ce qui permettra de supprimer les 
 * enregistrements non modifis apres le parsing
 *  
 */
void reset_changed() {
  int error = 0;
  
  if ( (error = mysql_query(&db_mysql, "truncate table l_temp")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update contacts set changed='0'") )!= 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update contactgroups set changed='0'") != 0 )) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update hosts set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update hostgroups set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update services set changed='0'")) != 0 ) { exit(error);}
  
  if ( (error = mysql_query(&db_mysql, "update l_hosts_hostgroups set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_services_hosts set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_contactgroups_hostgroups set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_contacts_contactgroups set changed='0'")) != 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_services_contactgroups set changed='0'")) != 0 ) { exit(error);}
}




/**
 * function make_links
 * 
 * rcupration des informations de la table l_temp
 * cration et insertion en base des liaisons entre les diffrents objets
 *  
 */
void make_links() {
  char * req = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW row;
  int contact_courant ;
  int cg_courant;
  char * host_name_courant ;
  int host_courant;
  int error;
  int hostgroup_courant ;
  char * hostgroup_name_courant ;
  if ( (error = mysql_query(&db_mysql, "update l_hosts_hostgroups set changed='0'"))!= 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_services_hosts set changed='0'"))!= 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_contactgroups_hostgroups set changed='0'"))!= 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_contacts_contactgroups set changed='0'"))!= 0 ) { exit(error);}
  if ( (error = mysql_query(&db_mysql, "update l_services_contactgroups set changed='0'"))!= 0 ) { exit(error);}
  

  //	l_contacts_contactgroups
  snprintf( req, 1000, "select value,id_temp from l_temp where type='contact' order by value");
#ifdef DEBUG
  printf( "%s\n", req); fflush( stdout) ;
#endif
  if ( (error = mysql_query(&db_mysql, req) ) != 0 ) { exit(error);}
  res = mysql_store_result(&db_mysql) ;
  contact_courant = 1;
  while (row = mysql_fetch_row(res)) {
    if (contact_courant != atoi(row[0])) {
      char * subquery = calloc(1000, sizeof(char));
      MYSQL_RES *subres = NULL;
      MYSQL_ROW contact;
      snprintf(subquery, 1000, "select id from contacts where name='%s'", row[0]);
#ifdef DEBUG
      printf( "%s\n", subquery);
#endif
      mysql_query(&db_mysql, subquery);
      subres = mysql_store_result(&db_mysql) ;
      
      contact = mysql_fetch_row(subres);
      contact_courant = atoi(contact[0]);
      free( subquery );
    }
    if (contact_courant != 0) {
      char * subquery = calloc(1000, sizeof(char));
      MYSQL_RES *subres = NULL;
      
      snprintf( subquery, 1000, "select * from l_contacts_contactgroups where id_contactgroups='%s' and id_contacts='%d'", row[1], contact_courant);
#ifdef DEBUG
      printf( "%s\n", subquery);
#endif
      mysql_query(&db_mysql, subquery);
      subres = mysql_store_result(&db_mysql);
      if (mysql_num_rows(subres) == 0) {
        char * subsubquery = calloc(1000, sizeof(char));
        snprintf( subsubquery, 1000, "insert into l_contacts_contactgroups (id_contactgroups, id_contacts, changed) values ('%s', '%d', '1')", row[1], contact_courant );
#ifdef DEBUG
        printf( "%s\n", subsubquery);
#endif
        mysql_query(&db_mysql,subsubquery );
        free( subsubquery );
      } else {
        char * subsubquery = calloc(1000, sizeof(char));
        snprintf( subsubquery, 1000, "update l_contacts_contactgroups set changed='1' where id_contactgroups='%s' and id_contacts='%d'", row[1], contact_courant );
#ifdef DEBUG
        printf( "%s\n", subsubquery);
#endif
        mysql_query(&db_mysql, subsubquery);
        free( subsubquery );
      }
    }
  }
  
  //  l_services_hosts
  snprintf( req, 1000, "select value,id_temp from l_temp where type='service' order by value");
#ifdef DEBUG
  printf( "%s\n", req); fflush( stdout);
#endif
  mysql_query(&db_mysql, req);
  res = mysql_store_result(&db_mysql) ;
  host_courant = 0;
  host_name_courant = calloc(1000, sizeof(char));
  
  while ( row = mysql_fetch_row(res)) {
#ifdef DEBUG
    printf("ROW   %s    host_name_courant: %s \n", row[0], host_name_courant );
#endif
    if ( strcmp(host_name_courant, row[0]) != 0 ) {
      char * subquery = calloc(1000, sizeof(char));
      MYSQL_RES *subres = NULL;
      MYSQL_ROW host;
      snprintf(subquery, 1000, "select id, name from hosts where name='%s'",row[0]);
      
#ifdef DEBUG
      printf( "1 %s\n", subquery); fflush( stdout);
#endif
      mysql_query(&db_mysql, subquery);

      subres = mysql_store_result(&db_mysql) ;

      host = mysql_fetch_row(subres);
      host_courant = atoi(host[0]);
      host_name_courant = host[1];
#ifdef DEBUG
      printf("host_courant: %d, host_name: %s\n", host_courant, host_name_courant);
#endif
    }
    if (host_courant != 0) {
      char * subquery = calloc(1000, sizeof(char));
      MYSQL_RES *subres = NULL;
      
      snprintf( subquery, 1000, "select * from l_services_hosts where id_services='%s' and id_hosts='%d'",
                row[1],
                host_courant
                );
#ifdef DEBUG
      printf( "%s\n", subquery); fflush( stdout);
#endif
      mysql_query(&db_mysql,subquery );
      subres = mysql_store_result(&db_mysql);
      if (mysql_num_rows(subres) == 0) {
        char * subsubquery = calloc(1000, sizeof(char));
        snprintf( subsubquery, 1000, "insert into l_services_hosts (id_services, id_hosts, changed) values ('%s', '%d', '2')", row[1], host_courant );
#ifdef DEBUG
        printf( "%s\n", subsubquery);
#endif
        mysql_query(&db_mysql,subsubquery );

        free( subquery );
      } 
      else {
        char * subsubquery = calloc(1000, sizeof(char));
        snprintf( subsubquery, 
                  1000,
                  "update l_services_hosts set changed='1' where id_services='%s' and id_hosts='%d'",
                  row[1],
                  host_courant
                  );
#ifdef DEBUG
        printf( "%s\n", subsubquery); fflush( stdout);
#endif
        mysql_query(&db_mysql, subsubquery);
        free( subsubquery );        
      }
    }
  }
  
  
  
}

/**
 * function recalcul_liaison
 * 
 * insre automatiquement un nouveau service nagios dans nareto en fonction 
 * du host contenant le service
 *  
 */
void recalcul_liaisons() {
  char * req = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW row;
  snprintf( req, 1000, "select id_hosts, id_services from l_services_hosts where changed='2'");
  mysql_query(&db_mysql, req);
  res = mysql_store_result(&db_mysql) ;
  while (row = mysql_fetch_row(res)) {
    char * req_2 = calloc(1000, sizeof(char));
    MYSQL_RES *res_2 = NULL;
    MYSQL_ROW row_2 = NULL;
    snprintf( req_2, 1000, "select distinct id_boite from l_boite_services where id_host='%s'", row[0]);
    mysql_query(&db_mysql, req_2);
    res_2 = mysql_store_result(&db_mysql) ;
    while (row_2 = mysql_fetch_row(res_2)) {
      char * req_3 = calloc(1000, sizeof(char));
      snprintf( req_3, 
                1000, 
                "insert into l_boite_services (id_boite, id_host, id_services, affiche) values ('%s', '%s', '%s', '1')",
                row_2[0],
                row[0],
                row[1]);
      mysql_query(&db_mysql, req_3);
      free ( req_3 );
    }
    free ( req_2 );
  } 
  
  free ( req );
}

/**
 * function delete_not_changed
 * 
 * suppression des enregistrements n'ayant t ni ajout ni modifi
 * ces enregistrements n'ont plus lieu d'etre car ils ne sont plus dans nagios 
 *  
 */
void delete_not_changed() {
  char * req = calloc(1000, sizeof(char));
  MYSQL_RES *res = NULL;
  MYSQL_ROW row;
  
  mysql_query(&db_mysql, "delete from contacts where changed='0'");
  mysql_query(&db_mysql, "delete from contactgroups where changed='0'");
  mysql_query(&db_mysql, "delete from hostgroups where changed='0'");
  mysql_query(&db_mysql, "delete from services where changed='0'");
  mysql_query(&db_mysql, "delete from l_hosts_hostgroups where changed='0'");

  mysql_query(&db_mysql, "select id from hosts where changed='0'");
#ifdef DEBUG
        printf( "%s\n","select id from hosts where changed='0'" );
#endif
  
  res = mysql_store_result(&db_mysql) ;
  while (row = mysql_fetch_row(res)) {
     MYSQL_RES *res_del = NULL;
     MYSQL_ROW row_del;
  
    req = calloc(1000, sizeof(char));
    snprintf(req, 
             1000, 
             "select id_boite from l_boite_services where id_host='%s'",
             row[0]);
#ifdef DEBUG
        printf( "%s\n", req);
#endif
    mysql_query(&db_mysql, req);
    res_del = mysql_store_result(&db_mysql) ;
    while (row_del = mysql_fetch_row(res_del)) {
      char * sub_req = calloc(1000, sizeof(char));
      snprintf(sub_req, 
               1000,
               "delete from boite where id='%s'",
               row_del[0]);
#ifdef DEBUG
        printf( "%s\n", sub_req);
#endif
      mysql_query(&db_mysql, sub_req);
      free( sub_req );
    }
    free( req );
    req = calloc(1000, sizeof(char));
    snprintf(req, 
             1000,
             "delete from l_boite_services where id_host='%s' and id_services='%s'",
             row[0],
             row[1]);
    mysql_query(&db_mysql, req);
    free( req );
  }
  
  mysql_query(&db_mysql, "delete from hosts where changed='0'");
	
  mysql_query(&db_mysql, "select id_hosts, id_services from l_services_hosts where changed='0'");
  
  res = mysql_store_result(&db_mysql) ;
  while (row = mysql_fetch_row(res)) {
     MYSQL_RES *res_del = NULL;
     MYSQL_ROW row_del;
  
    req = calloc(1000, sizeof(char));
    snprintf(req, 
             1000, 
             "select id_boite from l_boite_services where id_host='%s' and id_services='%s'",
             row[0],
             row[1]);
    mysql_query(&db_mysql, req);
    res_del = mysql_store_result(&db_mysql) ;
    while (row_del = mysql_fetch_row(res_del)) {
      char * sub_req = calloc(1000, sizeof(char));
      snprintf(sub_req, 
               1000,
               "delete from boite where id='%s'",
               row_del[0]);
      mysql_query(&db_mysql, sub_req);
      free( sub_req );
    }
    free( req );
    req = calloc(1000, sizeof(char));
    snprintf(req, 
             1000,
             "delete from l_boite_services where id_host='%s' and id_services='%s'",
             row[0],
             row[1]);
    mysql_query(&db_mysql, req);
    free( req );
  }

        
  mysql_query(&db_mysql, "delete from l_services_hosts where changed='0'");
  mysql_query(&db_mysql, "delete from l_contactgroups_hostgroups where changed='0'");
  mysql_query(&db_mysql, "delete from l_contacts_contactgroups where changed='0'");
  mysql_query(&db_mysql, "delete from l_services_contactgroups where changed='0'");
  mysql_query(&db_mysql, "delete from services where register=0");
  mysql_query(&db_mysql, "delete from hosts where register=0");
	
}


void print_help(char * name){
  printf("This file is part of the NaReTo project.\n");
  printf("\n");
  printf("Copyright (C) 2006/2007 Cedric TEMPLE Linagora SA\n");
  printf("\n");
  printf("This program is free software; you can redistribute it and/or modify\n");
  printf("it under the terms of the GNU General Public License as published by\n");
  printf("the Free Software Foundation; either version 2 of the License, or\n");
  printf("(at your option) any later version.\n");
  printf("\n");
  printf("This program is distributed in the hope that it will be useful,\n");
  printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
  printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
  printf("GNU General Public License for more details.\n");
  printf("\n");
  printf("You should have received a copy of the GNU General Public License\n");
  printf("along with this program; if not, write to the Free Software\n");
  printf("Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

  printf("\n\n\n");
  printf("USAGE: %s mysql_host  mysql_database  mysql_user mysql_password mysql_port\n\n",
         name);

  exit( 3 );

         
}
