/*
    parser_log.php

    This file is part of the NaReTo project.

    Copyright (C) 2006/2007 Cedric TEMPLE Linagora SA

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "../include/config.h"
#include "../include/locations.h"
#include "../include/common.h"
#include "../include/objects.h"
#include "../include/statusdata.h"

#include "../include/getcgi.h"
#include "../include/cgiutils.h"
#include "../include/cgiauth.h"


#include <mysql/mysql.h>

int update_hosts_status();
int update_services_status();
void print_help(char * );

extern char   main_config_file[MAX_FILENAME_LENGTH];

extern hostgroup *hostgroup_list;
extern hoststatus *hoststatus_list;
extern servicestatus *servicestatus_list;
extern host *host_list;
extern service *service_list;

int result;

MYSQL db_mysql;
char * query;

int main(int argc, char ** ARGV){

  char * mysql_database,*  mysql_host, * mysql_user, *mysql_password;
  int mysql_port;
  
  if ( argc != 6 ) {
    print_help( ARGV[0] );
  }

  query = calloc(1000, sizeof(char));

  /* read the CGI configuration file */
  result=read_cgi_config_file(get_cgi_config_location());
  if(result==ERROR){
    cgi_config_file_error(get_cgi_config_location());
    printf("Ne peut lire %s\n", get_cgi_config_location());
    return ERROR;
  }
  

  /* read the main configuration file */
  result=read_main_config_file(main_config_file);
  if(result==ERROR){
    printf("Ne peut lire %s\n", main_config_file);
    return ERROR;
  }
  
  /* read all object configuration data */
  result=read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);
  if(result==ERROR){
    printf("Ne peut lire toute la configuration\n");
    return ERROR;
  }

  /* read all status data */
  result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
  if(result==ERROR){
    printf("Ne peut lire tous les etats\n");
    fflush(stdout);
    free_memory();
    return ERROR;
  }

  /* try to initialize MySQL */
  if (!mysql_init (&db_mysql)) {
    printf( "Cannot initialise MySQL\n");
    return ERROR;
  }

  
  mysql_host =  (char * ) ARGV[1]; 
  mysql_database = (char * )ARGV[2];
  mysql_user = (char * )ARGV[3];
  mysql_password = (char * )ARGV[4];
  mysql_port = atoi( (char * )ARGV[5] );


  /* try to connect to MySQL */
  if (!mysql_real_connect(&db_mysql,  
                          mysql_host, 
                          mysql_user, 
                          mysql_password, 
                          mysql_database, 
                          mysql_port, 
                          NULL, 0)) {
    printf( "Can not connect to database \"%s\" on server \"%s:%d\", with user \"%s\" and password!\n",
            mysql_database,
            mysql_host, 
            mysql_port,
            mysql_user
            );
    return ERROR;    
  }

  /* update hosts status */
  result=update_hosts_status();
  if(result==ERROR){
    printf("Ne peut mettre a jour le statut des hosts en base de donnees\n");
    fflush(stdout);
    return ERROR;
  }


  /* update services_status */
  result=update_services_status();
  if(result==ERROR){
    printf("Ne peut mettre a jour le statut des services en base de donnees\n");
    fflush(stdout);
    return ERROR;
  }
  

}

int update_hosts_status(){
  // servicestatus *temp_servicestatus;
  // service *temp_service;
  hoststatus *temp_hoststatus;
  host *temp_host;
  char * nareto_host_status;
  
  for(temp_hoststatus=hoststatus_list;temp_hoststatus!=NULL;temp_hoststatus=temp_hoststatus->next){
    // temp_host=find_host(temp_hoststatus->host_name,NULL);
    switch( temp_hoststatus->status ) {
    case HOST_UP : 
      nareto_host_status = "UP"; break;
    case HOST_DOWN :
      nareto_host_status = "DOWN"; break;
    case HOST_PENDING :
      nareto_host_status = "PENDING"; break;
    case HOST_UNREACHABLE :
      nareto_host_status = "UNREACHABLE"; break;
    default:
      nareto_host_status = "UNKNOWN"; break;
    }
    snprintf( query, 1000, "UPDATE hosts SET statut='%s' WHERE name='%s'\n",nareto_host_status, temp_hoststatus->host_name);
    mysql_query(&db_mysql, query);
    if ( mysql_errno(&db_mysql) ) {
      printf( "QUERY: %s\n", query);
      printf( "MYSQL ERROR on HOST UPDATE: %s\n", mysql_error(&db_mysql));
    }
  }

  
  
}


int update_services_status(){
  servicestatus *temp_servicestatus;
  service *temp_service;
  hoststatus *temp_hoststatus;
  host *temp_host;
  int nareto_service_status;
  time_t now;
  int not_checked = 0;
  int is_in_downtime = 0;

  /* get the local time: used for not_checked */
  now=time(&now);
  
  for(temp_servicestatus=servicestatus_list;temp_servicestatus!=NULL;temp_servicestatus=temp_servicestatus->next){

    /* search for service, host and hoststatus */
    temp_service = find_service(temp_servicestatus->host_name,temp_servicestatus->description);
    temp_host = find_host( temp_servicestatus->host_name );
    temp_hoststatus = find_hoststatus( temp_servicestatus->host_name );

    /* translate Nagios status ---> NaReTo status */
    switch( temp_servicestatus->status ) {
    case SERVICE_OK :
      nareto_service_status = 1; break;
    case SERVICE_PENDING :
      nareto_service_status = 2; break;
    case SERVICE_UNKNOWN :
      nareto_service_status = 6; break;
    case SERVICE_WARNING :
      nareto_service_status = 7; break;
    case SERVICE_CRITICAL :
      nareto_service_status = 8; break;
    }

    /* checks if service is in check_period */
    if ( check_time_against_period(now, temp_service->check_period) == OK ) 
      not_checked = 0;
    else
      not_checked = 1;

     /* checks if service is in downtime */
    if ( ( temp_servicestatus->scheduled_downtime_depth >= 1 ) 
         ||  
         ( temp_hoststatus->scheduled_downtime_depth >= 1 ) 
         ) 
      is_in_downtime = 1;
    else
      is_in_downtime = 0;
    
    /* correct nareto_service_status if service is ACKed */
    if ( ( nareto_service_status >= 6 ) && ( temp_servicestatus->problem_has_been_acknowledged  ) ) {
      nareto_service_status = nareto_service_status - 3;
      /* extract comment ACK and people that acked */
      /* IT SHOULD BE NOT NECESSARY TO DONE THIS */
    }

    snprintf( query, 1000, "UPDATE l_services_hosts,hosts,services SET l_services_hosts.etat=%d, "
            "not_checked='%d', "
            "downtime='%d' "
            "WHERE hosts.name='%s' "
            "AND services.name='%s' "
            "AND l_services_hosts.id_services=services.id "
            "AND l_services_hosts.id_hosts=hosts.id \n "
            , nareto_service_status, not_checked, is_in_downtime, temp_servicestatus->host_name, temp_servicestatus->description);

    //printf(query);
    mysql_query(&db_mysql, query);
    if ( mysql_errno(&db_mysql) ) {
      printf( "QUERY: %s\n", query);
      printf( "MYSQL ERROR ON l_services_hosts UPDATE: %s\n", mysql_error(&db_mysql));
    }
    
    snprintf( query, 1000, "UPDATE l_boite_services,hosts,services SET etat=%d, "
            "not_checked='%d', "
            "downtime='%d' "
            "WHERE hosts.name='%s' "
            "AND services.name='%s' "
            "AND l_boite_services.id_services=services.id "
            "AND l_boite_services.id_host=hosts.id \n "
            , nareto_service_status, not_checked, is_in_downtime, temp_servicestatus->host_name, temp_servicestatus->description);
            
    //printf( query ) ;
    mysql_query(&db_mysql, query);
    if ( mysql_errno(&db_mysql) ) {
      printf( "QUERY: %s\n", query);
      printf( "MYSQL ERROR ON l_boite_services: %s\n", mysql_error(&db_mysql));
    }
  }

  
}


/* see if the specified time falls into a valid time range in the given time period */
int check_time_against_period(time_t check_time,char *period_name){
	timeperiod *temp_period;
	timerange *temp_range;
	unsigned long midnight_today;
	struct tm *t;

#ifdef DEBUG0
	printf("check_time_against_period() start\n");
#endif
	
	/* if no period name was specified, assume the time is good */
	if(period_name==NULL)
		return OK;

	/* if no period name was specified, assume the time is good */
	if(!strcmp(period_name,""))
		return OK;

	/* if period could not be found, assume the time is good */
	temp_period=find_timeperiod(period_name);
	if(temp_period==NULL)
		return OK;

	t=localtime((time_t *)&check_time);

	/* calculate the start of the day (midnight, 00:00 hours) */
	t->tm_sec=0;
	t->tm_min=0;
	t->tm_hour=0;
	midnight_today=(unsigned long)mktime(t);

	for(temp_range=temp_period->days[t->tm_wday];temp_range!=NULL;temp_range=temp_range->next){

		/* if the user-specified time falls in this range, return with a positive result */
		if(check_time>=(time_t)(midnight_today+temp_range->range_start) && check_time<=(time_t)(midnight_today+temp_range->range_end))
			return OK;
	        }

#ifdef DEBUG0
	printf("check_time_against_period() end\n");
#endif

	return ERROR;
        }



void print_help( char * name){
  printf("This file is part of the NaReTo project.\n");
  printf("\n");
  printf("Copyright (C) 2006/2007 Cedric TEMPLE Linagora SA\n");
  printf("\n");
  printf("This program is free software; you can redistribute it and/or modify\n");
  printf("it under the terms of the GNU General Public License as published by\n");
  printf("the Free Software Foundation; either version 2 of the License, or\n");
  printf("(at your option) any later version.\n");
  printf("\n");
  printf("This program is distributed in the hope that it will be useful,\n");
  printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
  printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
  printf("GNU General Public License for more details.\n");
  printf("\n");
  printf("You should have received a copy of the GNU General Public License\n");
  printf("along with this program; if not, write to the Free Software\n");
  printf("Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n");

  printf("\n\n\n");
  printf("USAGE: %s mysql_host  mysql_database  mysql_user mysql_password mysql_port\n\n",
         name);

  exit( 3 );
         
}
